#ifndef _HIF_CLASSES_H_
#define _HIF_CLASSES_H_
#include <SDL/SDL.h>
#include <bzlib.h>
#include "hif_questions.h"
class CHIF_Drop {
public:
Uint16 x,y1,y2,phase;char delta,damage,is_falling;
CHIF_Drop(){};
Sint16 *get_info(Sint16 *t);
void init(Uint16 x_,Uint16 y1_,Uint16 y2_,char delta_,char damage_);
void refresh(int x0,int y0);
void proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Thing {
public:
float x,y,vx,vy;
int no,cell;
char type,is_hurting,is_lonely,is_resting;
CHIF_Thing(){};
void refresh(int x0,int y0);
int proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Vent {
public:
Uint16 x,y,xs,ys,phase;char v;
CHIF_Vent(){};
void init(Uint16 x_,Uint16 y_,char v_);
void refresh(int x0,int y0,SDL_Surface *s);//s - vent surface
void proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_River {
public:
Uint16 x,y,xs,ys,phase;char v;
CHIF_River(){};
void init(Uint16 x_,Uint16 y_,char v_);
void refresh(int x0,int y0,SDL_Surface *s);//s - river surface
void proceed();
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Fireball {
public:
float x,y,direction;
char phase,hang;
CHIF_Fireball(){};
void init(float _x,float _y,float _direction);
void refresh(int x0,int y0);
int proceed(void);//0 - OK,-1 - to delete
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Bee {
public:
float x,y;char enabled;Sint16 beer;char state,enemy,firetime;float direction;char ph_left,is_right,beer_line;
CHIF_Bee(){};
void init(int x_,int y_);
void refresh(int x0,int y0);
void proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Line {
public:
int x,y;Uint16 phase,rc,gc,bc;Uint16 len;
CHIF_Line(){};
void init(int x_,int y_,Uint16 rc_,Uint16 gc_,Uint16 bc_,Uint16 len_);
void refresh(int x0,int y0);
int proceed(void);//0 - OK,-1 - to delete
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Block {
public:
int x1,y1,x2,y2;char type;Uint16 phase;
CHIF_Block(){};
void init(int x1_,int y1_,int x2_,int y2_,char type_);
void refresh(int x0,int y0);
void proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Air {
public:
int x,y;
CHIF_Air(){};
void init(int x_,int y_);
void refresh(int x0,int y0);
int proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
class CHIF_Enemy {
public:
int x,y,health;char type;Sint16 n_mines;
CHIF_Enemy(){};
virtual Sint16 *get_info(Sint16 *t)=0;
virtual void refresh(int x0,int y0)=0;
virtual void all_proceed(void)=0;
virtual void save(BZFILE *f)=0;
virtual void load(BZFILE *f)=0;
};
class CHIF_SmallEnemy : public CHIF_Enemy {
public:
Sint16 phase,n_traps,t_asleep,sleeptime;char state,health_line,dead,thing,no_things,is_hurt;
CHIF_SmallEnemy(){};
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void all_proceed(void);
virtual void proceed(void){};
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Ghost:public CHIF_SmallEnemy {
public:
Sint16 r_small,r_big,t_small,t_big,x0,y0;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Snail:public CHIF_SmallEnemy {
public:
Uint16 x1,x2;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Car:public CHIF_SmallEnemy {
public:
Uint16 x1,x2;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Gun:public CHIF_SmallEnemy {
public:
Uint16 num_shells;char shoottime,maxtime,v;int x1,y1,x2,y2;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Pank:public CHIF_SmallEnemy {
public:
char is_right;
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Flower:public CHIF_SmallEnemy {
public:
void create_flower(int x_,int y_);
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
};
class CHIF_Enemy_Droid:public CHIF_SmallEnemy {
public:
char _v,_z,is_falling,flowerleft;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_BigBee:public CHIF_SmallEnemy {
public:
unsigned char is_right,is_up;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_Enemy_Mad:public CHIF_SmallEnemy {
public:
char phase;
CHIF_QuestionMaker *maker;
virtual Sint16 *get_info(Sint16 *t);
virtual void refresh(int x0,int y0);
virtual void proceed(void);
virtual void save(BZFILE *f);
virtual void load(BZFILE *f);
};
class CHIF_ScTh {
public:
float x,y;
int x1,y1,w,h,cx,cy;
char image,is_out;
CHIF_ScTh(){};
void init(int x_,int y_,char image_,int x1_,int y1_,int w_,int h_);
void refresh(int x0,int y0);
int proceed(void);
void save(BZFILE *f);
void load(BZFILE *f);
};
#endif
