#include <SDL/SDL_mixer.h>
#include <SDL/SDL_mutex.h>
#include <SDL/SDL_thread.h>
#include "log.h"
#include <bzlib.h>
#include "images.h"
#include <string.h>
#include "misc.h"
#define FIFOSIZE 40
#define FIFO_PREREAD 35
typedef struct {
  float frame_time;
  char is_xored;
  int n_frames;
} VIDEO_HEADER;
typedef struct {
  Uint8 frames[FIFOSIZE][60][80][2];
  char break_asked,file_out;
  SDL_sem *refresh_allowed;
} VIDEO_FIFO;
static BZFILE *f;
static VIDEO_FIFO fifo;
static VIDEO_HEADER header;
static SDL_Surface *fonts[51];
static int video_threadfunc(void *data)
{
Uint8 frame[60][80][2],new_frame[60][80][2];
char stillworking=1,i,i2,i3,surf_no=0;
memset(frame,0,sizeof(frame));memset(new_frame,0,sizeof(new_frame));
while (stillworking==1){
  if (fifo.break_asked==1){stillworking=0;break;};
  while ((SDL_SemValue(fifo.refresh_allowed)>=FIFOSIZE-2)&&(fifo.break_asked==0));//why 2? find correct value.
  if (fifo.break_asked==1){stillworking=0;break;};
  if (BZ2_bzread(f,new_frame,sizeof(new_frame))<(int)sizeof(new_frame)){fifo.file_out=1;SDL_SemPost(fifo.refresh_allowed);stillworking=0;break;};
  if (header.is_xored==1)for (i=0;i<60;i++)for (i2=0;i2<80;i2++)for (i3=0;i3<2;i3++)
      {new_frame[i][i2][i3]^=frame[i][i2][i3];frame[i][i2][i3]=new_frame[i][i2][i3];};
  memcpy(fifo.frames[surf_no],new_frame,2*60*80);
  SDL_SemPost(fifo.refresh_allowed);surf_no++;surf_no%=FIFOSIZE;
};
return 0;
}
void showvideo(int no)
{
static char name[]="video/00.bz2",name_mp3[]="video/00.mp3";
int x,y;
#define NAMEOFFSET 6
Mix_Music *mp3;
char still_showing=1,no_fr=0;
SDL_Thread *thread;
SDL_Event event;
float ticks_start;

fprintf(LOG,"Playing video no %i:\n",no);
name[NAMEOFFSET]='0'+no/10;name[NAMEOFFSET+1]='0'+no%10;
name_mp3[NAMEOFFSET]='0'+no/10;name_mp3[NAMEOFFSET+1]='0'+no%10;
if ((f=BZ2_bzopen(name,"r"))==NULL){fprintf(LOG,"     File %s not found - no sound.\n",name);return;}else{fprintf(LOG,"     Video opened OK.\n");};
if (BZ2_bzread(f,&header,sizeof(header))!=sizeof(header)){fprintf(LOG,"     Header reading failed.\n");BZ2_bzclose(f);return;};
fifo.break_asked=fifo.file_out=0;
if ((mp3=Mix_LoadMUS(name_mp3))==NULL){fprintf(LOG,"     File %s not found - no sound.\n",name_mp3);}else{fprintf(LOG,"     Sound loaded OK.\n");};
fifo.refresh_allowed=SDL_CreateSemaphore(0);thread=SDL_CreateThread(video_threadfunc,NULL);
while (SDL_SemValue(fifo.refresh_allowed)<FIFO_PREREAD);
Mix_PlayMusic(mp3,0);ticks_start=SDL_GetTicks();
misc_clearevents();
while (still_showing==1){
if ((fifo.file_out==1)&&(SDL_SemValue(fifo.refresh_allowed)==0)){still_showing=0;break;};
SDL_SemWait(fifo.refresh_allowed);
if ((fifo.file_out==1)&&(SDL_SemValue(fifo.refresh_allowed)==0)){still_showing=0;break;};
for (y=0;y<60;y++)for (x=0;x<80;x++){
  Uint8 no=fifo.frames[no_fr][y][x][0]&127;
  int surf=2*((Uint16)fifo.frames[no_fr][y][x][1])+(fifo.frames[no_fr][y][x][0]/128);
  SDL_Rect r1={x*8,y*8,8,8},
      r2={8*(no%16),8*(no/16)+64*(surf%10),8,8};
  if (fonts[surf/10]!=NULL)SDL_BlitSurface(fonts[surf/10],&r2,screen,&r1);
};
SDL_Flip(screen);
no_fr++;no_fr%=FIFOSIZE;
while (SDL_PollEvent(&event))if ((event.type==SDL_KEYDOWN)||(event.type==SDL_MOUSEBUTTONDOWN)){
  still_showing=0;fifo.break_asked=1;break;};
while (SDL_GetTicks()<ticks_start+header.frame_time);ticks_start+=header.frame_time;
};
SDL_WaitThread(thread,NULL);SDL_DestroySemaphore(fifo.refresh_allowed);
BZ2_bzclose(f);if (name_mp3!=NULL){Mix_HaltMusic();Mix_FreeMusic(mp3);};
fprintf(LOG,"     Video played OK.\n");
}
void video_init(void)
{
#define SETCOL(a,b,c,d) {palette[a][0]=b;palette[a][1]=c;palette[a][2]=d;}
Uint8 palette[507][3];
Uint8 c1,c2,n2=1,n;
int x;

if (screen->flags&SDL_DOUBLEBUF)n2++;
SDL_SetColorKey(images[_IMG_aafont],SDL_SRCCOLORKEY|SDL_RLEACCEL,C_WHITE(images[_IMG_aafont]));
for (c1=0;c1<13;c1++)for (c2=0;c2<13;c2++)SETCOL(169*0+13*c1+c2,247,c1*19,c2*19);
for (c1=0;c1<13;c1++)for (c2=0;c2<13;c2++)SETCOL(169*1+13*c1+c2,c1*19,247,c2*19);
for (c1=0;c1<13;c1++)for (c2=0;c2<13;c2++)SETCOL(169*2+13*c1+c2,c1*19,c2*19,247);
for (x=0;x<507;x++){
  if (x%10==0)if ((fonts[x/10]=SDL_CreateRGBSurface(SDL_RLEACCEL|SDL_HWSURFACE,128,640,32,SURFACE_COLOURS))==NULL)
      {fprintf(LOG,"Error creating video font.\n");return;}else{SDL_FillRect(fonts[x/10],NULL,C_BLACK(fonts[x/10]));};
      SDL_Rect r1={0,(x%10)*64,128,64},r2=r1;
      SDL_FillRect(fonts[x/10],&r1,SDL_MapRGB(fonts[x/10]->format,palette[x][0],palette[x][1],palette[x][2]));
      SDL_BlitSurface(images[_IMG_aafont],NULL,fonts[x/10],&r2);
      if (x%40==0)for (n=0;n<n2;n++){SDL_Rect r1={100+x/40,110,1,10};SDL_FillRect(screen,&r1,C_WHITE(screen));SDL_Flip(screen);};
      };
}
void video_done(void)
{
int i;
for (i=0;i<51;i++)SDL_FreeSurface(fonts[i]);
}
